@@ -63,6 +63,7 @@ struct _TangleActorPrivate {
 	TangleSpacing margin;
 	gfloat max_width;
 	gfloat max_height;
+	gdouble aspect_ratio;
 	gdouble alignment_x;
 	gdouble alignment_y;
 
@@ -91,6 +92,7 @@ struct _TangleActorPrivate {
 	guint margin_set : 1;
 	guint max_width_set : 1;
 	guint max_height_set : 1;
+	guint aspect_ratio_set : 1;
 	guint interacting : 1;
 	guint interactive_size_multiplier_set : 1;
 	guint propagating_events : 1;
@@ -122,6 +124,8 @@ enum {
 	PROP_MAX_WIDTH_SET,
 	PROP_MAX_HEIGHT,
 	PROP_MAX_HEIGHT_SET,
+	PROP_ASPECT_RATIO,
+	PROP_ASPECT_RATIO_SET,
 	PROP_ALIGNMENT_X,
 	PROP_ALIGNMENT_Y,
 	PROP_ALIGNMENT_MODE,
@@ -522,6 +526,33 @@ void tangle_actor_set_max_size(TangleActor* actor, gfloat max_width, gfloat max_
 	g_object_thaw_notify(G_OBJECT(actor));
 }
 
+gdouble tangle_actor_get_aspect_ratio(TangleActor* actor) {
+
+	return actor->priv->aspect_ratio;
+}
+
+void tangle_actor_set_aspect_ratio(TangleActor* actor, gdouble aspect_ratio) {
+	if (actor->priv->aspect_ratio != aspect_ratio) {
+		actor->priv->aspect_ratio = aspect_ratio;
+		tangle_actor_set_aspect_ratio_set(actor, TRUE);
+		g_object_notify(G_OBJECT(actor), "aspect-ratio");
+		clutter_actor_queue_relayout(CLUTTER_ACTOR(actor));
+	}
+}
+
+gboolean tangle_actor_get_aspect_ratio_set(TangleActor* actor) {
+
+	return actor->priv->aspect_ratio_set;
+}
+
+void tangle_actor_set_aspect_ratio_set(TangleActor* actor, gboolean is_set) {
+	if (actor->priv->aspect_ratio_set != is_set) {
+		actor->priv->aspect_ratio_set = is_set;
+		g_object_notify(G_OBJECT(actor), "aspect-ratio-set");
+		clutter_actor_queue_relayout(CLUTTER_ACTOR(actor));
+	}
+}
+
 gdouble tangle_actor_get_alignment_x(TangleActor* actor) {
 
 	return actor->priv->alignment_x;
@@ -573,57 +604,68 @@ void tangle_actor_get_preferred_width(TangleActor* actor, gfloat for_height, gbo
 	gboolean natural_width_set;
 	TangleActorClass* actor_class;
 
-	g_object_get(actor, "min-width", &min_width, "min-width-set", &min_width_set, "natural-width", &natural_width, "natural-width-set", &natural_width_set, NULL);
-	if (min_width_set && min_width_p) {
-		*min_width_p = min_width;
-		min_width_p = NULL;
-	
-	}
-	if (natural_width_set && natural_width_p) {
-		*natural_width_p = natural_width;
-		natural_width_p = NULL;	
-	}
-	
-	if (min_width_p || natural_width_p || max_width_p) {
-		actor_class = TANGLE_ACTOR_GET_CLASS(actor);
-		if (actor_class->get_preferred_width) {
-			actor_class->get_preferred_width(actor, for_height, interacting, min_width_p, natural_width_p, max_width_p);
-
-			if (natural_width_p && min_width_p && *natural_width_p < *min_width_p) {
-				*natural_width_p = *min_width_p;
-			}
-			if (max_width_p && natural_width_p && *max_width_p != 0 && *max_width_p < *natural_width_p) {
-				*max_width_p = *natural_width_p;
-			}
-			if (actor->priv->interacting && actor->priv->interactive_size_multiplier_set) {
-				if (min_width_p) {
-					*min_width_p *= actor->priv->interactive_size_multiplier;
-				}
-				if (natural_width_p) {
-					*natural_width_p *= actor->priv->interactive_size_multiplier;
-				}
-				if (max_width_p) {
-					*max_width_p *= actor->priv->interactive_size_multiplier;
-				}
-			}
-		} else {
-			if (min_width_p) {
-				*min_width_p = 0.0;
-			}
-			if (natural_width_p) {
-				*natural_width_p = 0.0;
-			}
-			if (max_width_p) {
-				*max_width_p = 0.0;
-			}
+	if (for_height >= 0 && actor->priv->aspect_ratio_set) {
+		if (min_width_p) {
+			*min_width_p = actor->priv->aspect_ratio * for_height;
+		}
+		if (natural_width_p) {
+			*natural_width_p = actor->priv->aspect_ratio * for_height;
+		}
+		if (max_width_p) {
+			*max_width_p = actor->priv->aspect_ratio * for_height;
+		}
+	} else {
+		g_object_get(actor, "min-width", &min_width, "min-width-set", &min_width_set, "natural-width", &natural_width, "natural-width-set", &natural_width_set, NULL);
+		if (min_width_set && min_width_p) {
+			*min_width_p = min_width;
+			min_width_p = NULL;
+		}
+		if (natural_width_set && natural_width_p) {
+			*natural_width_p = natural_width;
+			natural_width_p = NULL;	
 		}
 
-		if (actor->priv->margin_set) {
-			if (*min_width_p) {
-				*min_width_p += actor->priv->margin.left + actor->priv->margin.right;
+		if (min_width_p || natural_width_p || max_width_p) {
+			actor_class = TANGLE_ACTOR_GET_CLASS(actor);
+			if (actor_class->get_preferred_width) {
+				actor_class->get_preferred_width(actor, for_height, interacting, min_width_p, natural_width_p, max_width_p);
+
+				if (natural_width_p && min_width_p && *natural_width_p < *min_width_p) {
+					*natural_width_p = *min_width_p;
+				}
+				if (max_width_p && natural_width_p && *max_width_p != 0 && *max_width_p < *natural_width_p) {
+					*max_width_p = *natural_width_p;
+				}
+				if (actor->priv->interacting && actor->priv->interactive_size_multiplier_set) {
+					if (min_width_p) {
+						*min_width_p *= actor->priv->interactive_size_multiplier;
+					}
+					if (natural_width_p) {
+						*natural_width_p *= actor->priv->interactive_size_multiplier;
+					}
+					if (max_width_p) {
+						*max_width_p *= actor->priv->interactive_size_multiplier;
+					}
+				}
+			} else {
+				if (min_width_p) {
+					*min_width_p = 0.0;
+				}
+				if (natural_width_p) {
+					*natural_width_p = 0.0;
+				}
+				if (max_width_p) {
+					*max_width_p = 0.0;
+				}
 			}
-			if (*natural_width_p) {
-				*natural_width_p += actor->priv->margin.left + actor->priv->margin.right;
+
+			if (actor->priv->margin_set) {
+				if (*min_width_p) {
+					*min_width_p += actor->priv->margin.left + actor->priv->margin.right;
+				}
+				if (*natural_width_p) {
+					*natural_width_p += actor->priv->margin.left + actor->priv->margin.right;
+				}
 			}
 		}
 	}
@@ -636,58 +678,69 @@ void tangle_actor_get_preferred_height(TangleActor* actor, gfloat for_width, gbo
 	gboolean natural_height_set;
 	TangleActorClass* actor_class;
 
-	g_object_get(actor, "min-height", &min_height, "min-height-set", &min_height_set, "natural-height", &natural_height, "natural-height-set", &natural_height_set, NULL);
-	if (min_height_set && min_height_p) {
-		*min_height_p = min_height;
-		min_height_p = NULL;
-	
-	}
-	if (natural_height_set && natural_height_p) {
-		*natural_height_p = natural_height;
-		natural_height_p = NULL;	
-	}
-	
-	if (min_height_p || natural_height_p || max_height_p) {
-		actor_class = TANGLE_ACTOR_GET_CLASS(actor);
-		if (actor_class->get_preferred_height) {
-			actor_class->get_preferred_height(actor, for_width, interacting, min_height_p, natural_height_p, max_height_p);
-
-			if (natural_height_p && min_height_p && *natural_height_p < *min_height_p) {
-				*natural_height_p = *min_height_p;
-			}
-			if (max_height_p && natural_height_p && *max_height_p != 0 && *max_height_p < *natural_height_p) {
-				*max_height_p = *natural_height_p;
-			}
-			if (actor->priv->interacting && actor->priv->interactive_size_multiplier_set) {
-				if (min_height_p) {
-					*min_height_p *= actor->priv->interactive_size_multiplier;
-				}
-				if (natural_height_p) {
-					*natural_height_p *= actor->priv->interactive_size_multiplier;
-				}
-				if (max_height_p) {
-					*max_height_p *= actor->priv->interactive_size_multiplier;
-				}
-			}
-		} else {
-			if (min_height_p) {
-				*min_height_p = 0.0;
-			}
-			if (natural_height_p) {
-				*natural_height_p = 0.0;
-			}
-			if (max_height_p) {
-				*max_height_p = 0.0;
-			}
+	if (for_width >= 0 && actor->priv->aspect_ratio_set) {
+		if (min_height_p) {
+			*min_height_p = for_width / actor->priv->aspect_ratio;
+		}
+		if (natural_height_p) {
+			*natural_height_p = for_width / actor->priv->aspect_ratio;
+		}
+		if (max_height_p) {
+			*max_height_p = for_width / actor->priv->aspect_ratio;
+		}
+	} else {
+		g_object_get(actor, "min-height", &min_height, "min-height-set", &min_height_set, "natural-height", &natural_height, "natural-height-set", &natural_height_set, NULL);
+		if (min_height_set && min_height_p) {
+			*min_height_p = min_height;
+			min_height_p = NULL;
+		}
+		if (natural_height_set && natural_height_p) {
+			*natural_height_p = natural_height;
+			natural_height_p = NULL;	
 		}
 
-		if (actor->priv->margin_set) {
-			if (*min_height_p) {
-				*min_height_p += actor->priv->margin.top + actor->priv->margin.bottom;
+		if (min_height_p || natural_height_p || max_height_p) {
+			actor_class = TANGLE_ACTOR_GET_CLASS(actor);
+			if (actor_class->get_preferred_height) {
+				actor_class->get_preferred_height(actor, for_width, interacting, min_height_p, natural_height_p, max_height_p);
+
+				if (natural_height_p && min_height_p && *natural_height_p < *min_height_p) {
+					*natural_height_p = *min_height_p;
+				}
+				if (max_height_p && natural_height_p && *max_height_p != 0 && *max_height_p < *natural_height_p) {
+					*max_height_p = *natural_height_p;
+				}
+				if (actor->priv->interacting && actor->priv->interactive_size_multiplier_set) {
+					if (min_height_p) {
+						*min_height_p *= actor->priv->interactive_size_multiplier;
+					}
+					if (natural_height_p) {
+						*natural_height_p *= actor->priv->interactive_size_multiplier;
+					}
+					if (max_height_p) {
+						*max_height_p *= actor->priv->interactive_size_multiplier;
+					}
+				}
+			} else {
+				if (min_height_p) {
+					*min_height_p = 0.0;
+				}
+				if (natural_height_p) {
+					*natural_height_p = 0.0;
+				}
+				if (max_height_p) {
+					*max_height_p = 0.0;
+				}
+			}
+
+			if (actor->priv->margin_set) {
+				if (*min_height_p) {
+					*min_height_p += actor->priv->margin.top + actor->priv->margin.bottom;
+				}
+				if (*natural_height_p) {
+					*natural_height_p += actor->priv->margin.top + actor->priv->margin.bottom;
+				}		
 			}
-			if (*natural_height_p) {
-				*natural_height_p += actor->priv->margin.top + actor->priv->margin.bottom;
-			}		
 		}
 	}
 }
@@ -859,7 +912,7 @@ static void set_max_width_set(TangleActor* actor, gboolean is_set) {
 }
 
 static void set_max_width(TangleActor* actor, gfloat max_width) {
-	if (actor->priv->max_width) {
+	if (actor->priv->max_width != max_width) {
 		g_object_freeze_notify(G_OBJECT(actor));
 	
 		actor->priv->max_width = max_width;
@@ -881,7 +934,7 @@ static void set_max_height_set(TangleActor* actor, gboolean is_set) {
 }
 
 static void set_max_height(TangleActor* actor, gfloat max_height) {
-	if (actor->priv->max_height) {
+	if (actor->priv->max_height != max_height) {
 		g_object_freeze_notify(G_OBJECT(actor));
 	
 		actor->priv->max_height = max_height;
@@ -909,6 +962,7 @@ static void tangle_actor_allocate(ClutterActor* self, const ClutterActorBox* box
 	gfloat length;
 	gfloat difference;
 	ClutterTimeline* timeline;
+	gdouble aspect_ratio;
 
 	actor = TANGLE_ACTOR(self);
 	if (TANGLE_ACTOR_IS_ALLOCATED(actor)) {
@@ -961,6 +1015,21 @@ static void tangle_actor_allocate(ClutterActor* self, const ClutterActorBox* box
 		}
 	}
 	
+	if (actor->priv->aspect_ratio_set &&
+	    (aspect_ratio = (actor->priv->aligned_allocation.x2 - actor->priv->aligned_allocation.x1) / (actor->priv->aligned_allocation.y2 - actor->priv->aligned_allocation.y1)) != actor->priv->aspect_ratio) {
+		if (aspect_ratio > actor->priv->aspect_ratio) {
+			length = actor->priv->aligned_allocation.x2 - actor->priv->aligned_allocation.x1;
+			difference = length - actor->priv->aspect_ratio * (actor->priv->aligned_allocation.y2 - actor->priv->aligned_allocation.y1);
+			actor->priv->aligned_allocation.x1 += difference * actor->priv->alignment_x;
+			actor->priv->aligned_allocation.x2 -= difference * (1 - actor->priv->alignment_x);
+		} else {
+			length = actor->priv->aligned_allocation.y2 - actor->priv->aligned_allocation.y1;
+			difference = length - actor->priv->aspect_ratio * (actor->priv->aligned_allocation.x2 - actor->priv->aligned_allocation.x1);
+			actor->priv->aligned_allocation.y1 += difference * actor->priv->alignment_y;
+			actor->priv->aligned_allocation.y2 -= difference * (1 - actor->priv->alignment_y);
+		}
+	}
+
 	set_scale_centers_by_gravity(actor);
 
 	if (!old_allocation ||
@@ -1300,6 +1369,12 @@ static void tangle_actor_set_property(GObject* object, guint prop_id, const GVal
 			actor->priv->squint_y = g_value_get_float(value);
 			clutter_actor_queue_redraw(CLUTTER_ACTOR(actor));
 			break;
+		case PROP_ASPECT_RATIO:
+			tangle_actor_set_aspect_ratio(actor, g_value_get_double(value));
+			break;
+		case PROP_ASPECT_RATIO_SET:
+			tangle_actor_set_aspect_ratio_set(actor, g_value_get_boolean(value));
+			break;
 		case PROP_INTERACTING:
 			tangle_actor_set_interacting(actor, g_value_get_boolean(value));
 			break;
@@ -1398,6 +1473,12 @@ static void tangle_actor_get_property(GObject* object, guint prop_id, GValue* va
 		case PROP_SQUINT_Y:
 			g_value_set_float(value, actor->priv->squint_y);
 			break;
+		case PROP_ASPECT_RATIO:
+			g_value_set_double(value, actor->priv->aspect_ratio);
+			break;
+		case PROP_ASPECT_RATIO_SET:
+			g_value_set_boolean(value, actor->priv->aspect_ratio_set);
+			break;
 		case PROP_INTERACTING:
 			g_value_set_boolean(value, actor->priv->interacting);
 			break;
@@ -1694,6 +1775,24 @@ static void tangle_actor_class_init(TangleActorClass* actor_class) {
 	                                                   "The vertical squint (drawing offset)",
 	                                                   -G_MAXFLOAT, G_MAXFLOAT, 0.0,
 	                                                   G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+	/**
+	 * TangleActor:aspect-ratio:
+	 */
+	g_object_class_install_property(gobject_class, PROP_ASPECT_RATIO,
+	                                g_param_spec_double("aspect-ratio",
+	                                                    "Aspect ratio",
+	                                                    "The fixed ratio of width / height",
+	                                                    0.0, G_MAXFLOAT, 1.0,
+	                                                    G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+	/**
+	 * TangleActor:aspect-ratio-set:
+	 */
+	g_object_class_install_property(gobject_class, PROP_ASPECT_RATIO_SET,
+	                                g_param_spec_boolean("aspect-ratio-set",
+	                                                     "Aspect ratio set",
+	                                                     "Whether the aspect ratio property is set or not",
+	                                                     FALSE,
+	                                                     G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 	/**
 	 * TangleWidget:interacting:
 	 */
@@ -1864,6 +1963,9 @@ static void tangle_actor_init(TangleActor* actor) {
 	actor->priv->transition_scale_gravity = CLUTTER_GRAVITY_CENTER;
 	actor->priv->picking_scale_x = 1.0;
 	actor->priv->picking_scale_y = 1.0;
+	actor->priv->aspect_ratio = 1.0;
+	actor->priv->alignment_x = 0.5;
+	actor->priv->alignment_y = 0.5;
 
 	actor->priv->animations = g_hash_table_new_full(g_str_hash, g_str_equal, (GDestroyNotify)g_free, (GDestroyNotify)g_object_unref);
 
