@@ -111,6 +111,11 @@ void tangle_actor_set_margin_set(TangleActor* actor, gboolean is_set);
 void tangle_actor_get_max_size(TangleActor* actor, gfloat* max_width_p, gfloat* max_height_p);
 void tangle_actor_set_max_size(TangleActor* actor, gfloat max_width, gfloat max_height);
 
+gdouble tangle_actor_get_aspect_ratio(TangleActor* actor);
+void tangle_actor_set_aspect_ratio(TangleActor* actor, gdouble aspect_ratio);
+gboolean tangle_actor_get_aspect_ratio_set(TangleActor* actor);
+void tangle_actor_set_aspect_ratio_set(TangleActor* actor, gboolean is_set);
+
 gdouble tangle_actor_get_alignment_x(TangleActor* actor);
 void tangle_actor_set_alignment_x(TangleActor* actor, gdouble alignment_x);
 gdouble tangle_actor_get_alignment_y(TangleActor* actor);
