@@ -446,22 +446,20 @@ static void start_interacting(TangleButton* button) {
 	TangleEvent* event;
 
 	tangle_actor_set_interacting(TANGLE_ACTOR(button), TRUE);
+	tangle_actor_send_interaction_began_event(TANGLE_ACTOR(button), TRUE, TRUE, FALSE);
 	if (button->priv->interactive_background_actor) {
 		tangle_widget_change_background_actor(TANGLE_WIDGET(button), button->priv->interactive_background_actor);
 	}
 	button->priv->button_release_event_handler_id = g_signal_connect(clutter_actor_get_stage(CLUTTER_ACTOR(button)), "button-release-event", G_CALLBACK(on_button_release_event), button);
 
-	event = tangle_event_new(TANGLE_EVENT_INTERACTION_BEGAN, TANGLE_ACTOR(button));
-	event->interaction.x_axis = TRUE;
-	event->interaction.y_axis = TRUE;
-	tangle_actor_send_event(TANGLE_ACTOR(button), event);
-	tangle_event_free(event);
+	tangle_actor_send_interaction_began_event(TANGLE_ACTOR(button), TRUE, TRUE, FALSE);
 }
 
 static void end_interacting(TangleButton* button) {
 	TangleEvent* event;
 	
 	tangle_actor_set_interacting(TANGLE_ACTOR(button), FALSE);
+	tangle_actor_send_interaction_ended_event(TANGLE_ACTOR(button), TRUE, TRUE, FALSE);
 	if (button->priv->mode == TANGLE_BUTTON_SELECTABLE && button->priv->selected && button->priv->selected_background_actor) {
 		tangle_widget_change_background_actor(TANGLE_WIDGET(button), button->priv->selected_background_actor);
 	} else {
@@ -472,11 +470,7 @@ static void end_interacting(TangleButton* button) {
 		button->priv->button_release_event_handler_id = 0;
 	}
 
-	event = tangle_event_new(TANGLE_EVENT_INTERACTION_ENDED, TANGLE_ACTOR(button));
-	event->interaction.x_axis = TRUE;
-	event->interaction.y_axis = TRUE;
-	tangle_actor_send_event(TANGLE_ACTOR(button), event);
-	tangle_event_free(event);
+	tangle_actor_send_interaction_ended_event(TANGLE_ACTOR(button), TRUE, TRUE, FALSE);
 }
 
 static gboolean on_button_release_event(ClutterActor* actor, ClutterEvent* event, gpointer user_data) {
