@@ -458,7 +458,7 @@ static void on_stage_paint(ClutterActor* actor, gpointer user_data) {
 	has_clip = clutter_actor_has_clip(actor);
 	if (has_clip) {
 		clutter_actor_get_clip(actor, &x, &y, &width, &height);
-		cogl_clip_push(x, y, width, height);
+		cogl_clip_push_rectangle(x, y, x + width, y + height);
 	}
 	
 	cogl_translate(floating_actor->priv->floating_x, floating_actor->priv->floating_y, 0.0);
