@@ -171,17 +171,21 @@ static void tangle_wrapper_actor_allocate(ClutterActor* actor, const ClutterActo
 	TANGLE_WRAPPER_ACTOR_GET_CLASS(wrapper_actor)->allocate_wrapped(wrapper_actor, wrapper_actor->priv->wrapped, &actor_box, flags);
 }
 
-static void tangle_wrapper_actor_paint(ClutterActor* actor) {
-	gdouble scale_x;
-	gdouble scale_y;
+static void tangle_wrapper_actor_paint_wrapped(TangleWrapperActor* wrapper_actor, ClutterActor* wrapped) {
+		clutter_actor_paint(wrapper_actor->priv->wrapped);
+}
 
-	if (CLUTTER_ACTOR_IS_MAPPED(TANGLE_WRAPPER_ACTOR(actor)->priv->wrapped)) {
+static void tangle_wrapper_actor_paint(ClutterActor* actor) {
+	TangleWrapperActor* wrapper_actor;
+	
+	wrapper_actor = TANGLE_WRAPPER_ACTOR(actor);
+
+	if (CLUTTER_ACTOR_IS_MAPPED(wrapper_actor->priv->wrapped)) {
 		cogl_push_matrix();
 
-		g_object_get(actor, "transition-scale-x", &scale_x, "transition-scale-y", &scale_y, NULL);
-		cogl_scale(scale_x, scale_y, 0);
+		tangle_actor_apply_transition_scale(TANGLE_ACTOR(wrapper_actor));
 
-		clutter_actor_paint(TANGLE_WRAPPER_ACTOR(actor)->priv->wrapped);
+		TANGLE_WRAPPER_ACTOR_GET_CLASS(wrapper_actor)->paint_wrapped(wrapper_actor, wrapper_actor->priv->wrapped);
 
 		cogl_pop_matrix();
 	}
@@ -275,6 +279,7 @@ static void tangle_wrapper_actor_class_init(TangleWrapperActorClass* wrapper_act
 	actor_class->handle_ancestor_event = tangle_wrapper_actor_handle_ancestor_event;
 
 	wrapper_actor_class->allocate_wrapped = tangle_wrapper_actor_allocate_wrapped;
+	wrapper_actor_class->paint_wrapped = tangle_wrapper_actor_paint_wrapped;
 
 	/**
 	 * TangleWrapperActor:actor:
@@ -300,15 +305,17 @@ static void tangle_wrapper_actor_set_custom_property(ClutterScriptable* scriptab
 
 	wrapper_actor = TANGLE_WRAPPER_ACTOR(scriptable);
 
-	if ((param_spec = g_object_class_find_property(G_OBJECT_GET_CLASS(G_OBJECT(wrapper_actor)), name))) {
-		object = G_OBJECT(wrapper_actor);
-	} else if ((param_spec = g_object_class_find_property(G_OBJECT_GET_CLASS(G_OBJECT(wrapper_actor->priv->wrapped)), name))) {
-		object = G_OBJECT(wrapper_actor->priv->wrapped);
-	} else {
-		g_warning("%s: Neither wrapper nor wrapped actor has property named '%s'.", G_STRLOC, name);
-	}
-	if (object) {
-		g_object_set_property(object, name, value);
+	if (name[0] != '$' && name[0] != '@') {
+		if ((param_spec = g_object_class_find_property(G_OBJECT_GET_CLASS(G_OBJECT(wrapper_actor)), name))) {
+			object = G_OBJECT(wrapper_actor);
+		} else if ((param_spec = g_object_class_find_property(G_OBJECT_GET_CLASS(G_OBJECT(wrapper_actor->priv->wrapped)), name))) {
+			object = G_OBJECT(wrapper_actor->priv->wrapped);
+		} else {
+			g_warning("%s: Neither wrapper nor wrapped actor has property named '%s'.", G_STRLOC, name);
+		}
+		if (object) {
+			g_object_set_property(object, name, value);
+		}
 	}
 }
 
