@@ -9,6 +9,10 @@ gboolean button2_clicked(TangleButton* button, gpointer user_data) {
 	g_printf("Button 2 clicked.\n");
 }
 
+void on_notify(GObject* object, GParamSpec* param_spec, gpointer user_data) {
+	g_printf("Notify: %s\n", g_param_spec_get_name(param_spec));
+}
+
 int main(int argc, char** argv) {
 	ClutterActor* stage;
 	const gchar* filename;
